import contextlib
import os
import struct
import sys
import tempfile
import uuid

from macholib import MachO, mach_o

if sys.version_info[:2] <= (2, 6):
    import unittest2 as unittest
else:
    import unittest


@contextlib.contextmanager
def temporary_macho_file(load_commands):
    struct_mach_header_64_format = ">IIIIIIII"
    cpu_type_arm64 = 0x100000C
    cpu_subtype_arm_all = 0x0
    mh_filetype_execute = 0x2
    ncmds = len(load_commands)
    sizeofcmds = sum([len(lc) for lc in load_commands])
    mach_header = struct.pack(
        struct_mach_header_64_format,
        mach_o.MH_MAGIC_64,
        cpu_type_arm64,
        cpu_subtype_arm_all,
        mh_filetype_execute,
        ncmds,
        sizeofcmds,
        0,
        0,
    )
    with tempfile.NamedTemporaryFile(delete=False) as macho_file:
        macho_file.write(mach_header)
        for lc in load_commands:
            macho_file.write(lc)
        # Close the file so it can be re-opened on Windows.
        macho_file.close()
        yield macho_file.name
        os.unlink(macho_file.name)


def lc_uuid(macho_uuid):
    lc_uuid_format = ">II16s"
    lc_uuid_size = struct.calcsize(lc_uuid_format)
    return struct.pack(lc_uuid_format, mach_o.LC_UUID, lc_uuid_size, macho_uuid.bytes)


def lc_unknown():
    lc_unknown_format = ">III"
    lc_unknown = 0x707A11ED  # Made-up load command. Hopefully never used.
    lc_unknown_size = struct.calcsize(lc_unknown_format)
    lc_unknown_value = 42  # Random value
    return struct.pack(lc_unknown_format, lc_unknown, lc_unknown_size, lc_unknown_value)


class TestMachO(unittest.TestCase):
    def test_known_load_command_should_succeed(self):
        macho_uuid = uuid.UUID("6894C0AE-C8B7-4E0B-A529-30BBEBA3703B")
        with temporary_macho_file([lc_uuid(macho_uuid)]) as macho_filename:
            macho = MachO.MachO(macho_filename, allow_unknown_load_commands=True)
            self.assertEqual(len(macho.headers), 1)
            self.assertEqual(len(macho.headers[0].commands), 1)
            load_command, command, _ = macho.headers[0].commands[0]
            self.assertEqual(load_command.cmd, mach_o.LC_UUID)
            self.assertEqual(uuid.UUID(bytes=command.uuid), macho_uuid)

    def test_unknown_load_command_should_fail(self):
        with temporary_macho_file([lc_unknown()]) as macho_filename:
            with self.assertRaises(ValueError) as assert_context:
                MachO.MachO(macho_filename)

    def test_unknown_load_command_should_succeed_with_flag(self):
        with temporary_macho_file([lc_unknown()]) as macho_filename:
            macho = MachO.MachO(macho_filename, allow_unknown_load_commands=True)
            self.assertEqual(len(macho.headers), 1)
            self.assertEqual(len(macho.headers[0].commands), 1)
            load_command, command, data = macho.headers[0].commands[0]
            self.assertEqual(load_command.cmd, 0x707A11ED)
            self.assertIsInstance(command, mach_o.load_command)
            self.assertEqual(struct.unpack(">I", data), (42,))

    def test_mix_of_known_and_unknown_load_commands_should_allow_unknown_with_flag(
        self,
    ):
        macho_uuid = uuid.UUID("6894C0AE-C8B7-4E0B-A529-30BBEBA3703B")
        with temporary_macho_file(
            [lc_unknown(), lc_uuid(macho_uuid)]
        ) as macho_filename:
            macho = MachO.MachO(macho_filename, allow_unknown_load_commands=True)
            self.assertEqual(len(macho.headers), 1)
            self.assertEqual(len(macho.headers[0].commands), 2)
            load_command, command, data = macho.headers[0].commands[0]
            self.assertEqual(load_command.cmd, 0x707A11ED)
            self.assertIsInstance(command, mach_o.load_command)
            self.assertEqual(struct.unpack(">I", data), (42,))
            load_command, command, _ = macho.headers[0].commands[1]
            self.assertEqual(load_command.cmd, mach_o.LC_UUID)
            self.assertEqual(uuid.UUID(bytes=command.uuid), macho_uuid)


if __name__ == "__main__":
    unittest.main()



import sys

from macholib import SymbolTable

if sys.version_info[:2] <= (2, 6):
    import unittest2 as unittest
else:
    import unittest


class TestSymbolTable(unittest.TestCase):
    @unittest.expectedFailure
    def test_missing(self):
        self.fail("tests are missing")


if __name__ == "__main__":
    unittest.main()



import sys

from macholib import dylib

if sys.version_info[:2] <= (2, 6):
    import unittest2 as unittest
else:
    import unittest


def d(location=None, name=None, shortname=None, version=None, suffix=None):
    return dict(
        location=location,
        name=name,
        shortname=shortname,
        version=version,
        suffix=suffix,
    )


class TestDylib(unittest.TestCase):
    def testInvalid(self):
        self.assertTrue(dylib.dylib_info("completely/invalid") is None)
        self.assertTrue(dylib.dylib_info("completely/invalid_debug") is None)

    def testUnversioned(self):
        self.assertEqual(dylib.dylib_info("P/Foo.dylib"), d("P", "Foo.dylib", "Foo"))
        self.assertEqual(
            dylib.dylib_info("P/Foo_debug.dylib"),
            d("P", "Foo_debug.dylib", "Foo", suffix="debug"),
        )

    def testVersioned(self):
        self.assertEqual(
            dylib.dylib_info("P/Foo.A.dylib"), d("P", "Foo.A.dylib", "Foo", "A")
        )
        self.assertEqual(
            dylib.dylib_info("P/Foo_debug.A.dylib"),
            d("P", "Foo_debug.A.dylib", "Foo_debug", "A"),
        )
        self.assertEqual(
            dylib.dylib_info("P/Foo.A_debug.dylib"),
            d("P", "Foo.A_debug.dylib", "Foo", "A", "debug"),
        )


if __name__ == "__main__":
    unittest.main()



import os
import shutil
import stat
import struct
import sys

from macholib import mach_o

MAGIC = [
    struct.pack("!L", getattr(mach_o, "MH_" + _))
    for _ in ["MAGIC", "CIGAM", "MAGIC_64", "CIGAM_64"]
]
FAT_MAGIC_BYTES = struct.pack("!L", mach_o.FAT_MAGIC)
MAGIC_LEN = 4
STRIPCMD = ["/usr/bin/strip", "-x", "-S", "-"]

try:
    unicode
except NameError:
    unicode = str


def fsencoding(s, encoding=sys.getfilesystemencoding()):  # noqa: M511,B008
    """
    Ensure the given argument is in filesystem encoding (not unicode)
    """
    if isinstance(s, unicode):
        s = s.encode(encoding)
    return s


def move(src, dst):
    """
    move that ensures filesystem encoding of paths
    """
    shutil.move(fsencoding(src), fsencoding(dst))


def copy2(src, dst):
    """
    copy2 that ensures filesystem encoding of paths
    """
    shutil.copy2(fsencoding(src), fsencoding(dst))


def flipwritable(fn, mode=None):
    """
    Flip the writability of a file and return the old mode. Returns None
    if the file is already writable.
    """
    if os.access(fn, os.W_OK):
        return None
    old_mode = os.stat(fn).st_mode
    os.chmod(fn, stat.S_IWRITE | old_mode)
    return old_mode


class fileview(object):
    """
    A proxy for file-like objects that exposes a given view of a file
    """

    def __init__(self, fileobj, start, size):
        self._fileobj = fileobj
        self._start = start
        self._end = start + size

    def __repr__(self):
        return "<fileview [%d, %d] %r>" % (self._start, self._end, self._fileobj)

    def tell(self):
        return self._fileobj.tell() - self._start

    def _checkwindow(self, seekto, op):
        if not (self._start <= seekto <= self._end):
            raise IOError(
                "%s to offset %d is outside window [%d, %d]"
                % (op, seekto, self._start, self._end)
            )

    def seek(self, offset, whence=0):
        seekto = offset
        if whence == 0:
            seekto += self._start
        elif whence == 1:
            seekto += self._fileobj.tell()
        elif whence == 2:
            seekto += self._end
        else:
            raise IOError("Invalid whence argument to seek: %r" % (whence,))
        self._checkwindow(seekto, "seek")
        self._fileobj.seek(seekto)

    def write(self, bytes):
        here = self._fileobj.tell()
        self._checkwindow(here, "write")
        self._checkwindow(here + len(bytes), "write")
        self._fileobj.write(bytes)

    def read(self, size=sys.maxsize):
        if size < 0:
            raise ValueError(
                "Invalid size %s while reading from %s", size, self._fileobj
            )
        here = self._fileobj.tell()
        self._checkwindow(here, "read")
        bytes = min(size, self._end - here)
        return self._fileobj.read(bytes)


def mergecopy(src, dest):
    """
    copy2, but only if the destination isn't up to date
    """
    if os.path.exists(dest) and os.stat(dest).st_mtime >= os.stat(src).st_mtime:
        return

    copy2(src, dest)


def mergetree(src, dst, condition=None, copyfn=mergecopy, srcbase=None):
    """
    Recursively merge a directory tree using mergecopy().
    """
    src = fsencoding(src)
    dst = fsencoding(dst)
    if srcbase is None:
        srcbase = src
    names = map(fsencoding, os.listdir(src))
    try:
        os.makedirs(dst)
    except OSError:
        pass
    errors = []
    for name in names:
        srcname = os.path.join(src, name)
        dstname = os.path.join(dst, name)
        if condition is not None and not condition(srcname):
            continue
        try:
            if os.path.islink(srcname):
                realsrc = os.readlink(srcname)
                os.symlink(realsrc, dstname)
            elif os.path.isdir(srcname):
                mergetree(
                    srcname,
                    dstname,
                    condition=condition,
                    copyfn=copyfn,
                    srcbase=srcbase,
                )
            else:
                copyfn(srcname, dstname)
        except (IOError, os.error) as why:
            errors.append((srcname, dstname, why))
    if errors:
        raise IOError(errors)


def sdk_normalize(filename):
    """
    Normalize a path to strip out the SDK portion, normally so that it
    can be decided whether it is in a system path or not.
    """
    if filename.startswith("/Developer/SDKs/"):
        pathcomp = filename.split("/")
        del pathcomp[1:4]
        filename = "/".join(pathcomp)
    return filename


NOT_SYSTEM_FILES = []


def in_system_path(filename):
    """
    Return True if the file is in a system path
    """
    fn = sdk_normalize(os.path.realpath(filename))
    if fn.startswith("/usr/local/"):
        return False
    elif fn.startswith("/System/") or fn.startswith("/usr/"):
        if fn in NOT_SYSTEM_FILES:
            return False
        return True
    else:
        return False


def has_filename_filter(module):
    """
    Return False if the module does not have a filename attribute
    """
    return getattr(module, "filename", None) is not None


def get_magic():
    """
    Get a list of valid Mach-O header signatures, not including the fat header
    """
    return MAGIC


def is_platform_file(path):
    """
    Return True if the file is Mach-O
    """
    if not os.path.exists(path) or os.path.islink(path):
        return False
    # If the header is fat, we need to read into the first arch
    with open(path, "rb") as fileobj:
        bytes = fileobj.read(MAGIC_LEN)
        if bytes == FAT_MAGIC_BYTES:
            # Read in the fat header
            fileobj.seek(0)
            header = mach_o.fat_header.from_fileobj(fileobj, _endian_=">")
            if header.nfat_arch < 1:
                return False
            # Read in the first fat arch header
            arch = mach_o.fat_arch.from_fileobj(fileobj, _endian_=">")
            fileobj.seek(arch.offset)
            # Read magic off the first header
            bytes = fileobj.read(MAGIC_LEN)
    for magic in MAGIC:
        if bytes == magic:
            return True
    return False


def iter_platform_files(dst):
    """
    Walk a directory and yield each full path that is a Mach-O file
    """
    for root, _dirs, files in os.walk(dst):
        for fn in files:
            fn = os.path.join(root, fn)
            if is_platform_file(fn):
                yield fn


def strip_files(files, argv_max=(256 * 1024)):
    """
    Strip a list of files
    """
    tostrip = [(fn, flipwritable(fn)) for fn in files]
    while tostrip:
        cmd = list(STRIPCMD)
        flips = []
        pathlen = sum(len(s) + 1 for s in cmd)
        while pathlen < argv_max:
            if not tostrip:
                break
            added, flip = tostrip.pop()
            pathlen += len(added) + 1
            cmd.append(added)
            flips.append((added, flip))
        else:
            cmd.pop()
            tostrip.append(flips.pop())
        os.spawnv(os.P_WAIT, cmd[0], cmd)
        for args in flips:
            flipwritable(*args)



import sys

from macholib import framework

if sys.version_info[:2] <= (2, 6):
    import unittest2 as unittest
else:
    import unittest


class TestFramework(unittest.TestCase):
    def test_framework(self):
        self.assertEqual(
            framework.framework_info(
                "Location/Name.framework/Versions/SomeVersion/Name_Suffix"
            ),
            dict(
                location="Location",
                name="Name.framework/Versions/SomeVersion/Name_Suffix",
                shortname="Name",
                version="SomeVersion",
                suffix="Suffix",
            ),
        )
        self.assertEqual(
            framework.framework_info(
                "Location/Name.framework/Versions/SomeVersion/Name"
            ),
            dict(
                location="Location",
                name="Name.framework/Versions/SomeVersion/Name",
                shortname="Name",
                version="SomeVersion",
                suffix=None,
            ),
        )
        self.assertEqual(
            framework.framework_info("Location/Name.framework/Name_Suffix"),
            dict(
                location="Location",
                name="Name.framework/Name_Suffix",
                shortname="Name",
                version=None,
                suffix="Suffix",
            ),
        )
        self.assertEqual(
            framework.framework_info("Location/Name.framework/Name"),
            dict(
                location="Location",
                name="Name.framework/Name",
                shortname="Name",
                version=None,
                suffix=None,
            ),
        )
        self.assertEqual(
            framework.framework_info("Location/Name.framework.disabled/Name"), None
        )
        self.assertEqual(
            framework.framework_info("Location/Name.framework/Versions/A/B/Name"), None
        )
        self.assertEqual(
            framework.framework_info("Location/Name.framework/Versions/A"), None
        )
        self.assertEqual(
            framework.framework_info("Location/Name.framework/Versions/A/Name/_debug"),
            None,
        )

    def test_interal_tests(self):
        # Ported over from the source file
        def d(location=None, name=None, shortname=None, version=None, suffix=None):
            return dict(
                location=location,
                name=name,
                shortname=shortname,
                version=version,
                suffix=suffix,
            )

        self.assertEqual(framework.framework_info("completely/invalid"), None)
        self.assertEqual(framework.framework_info("completely/invalid/_debug"), None)
        self.assertEqual(framework.framework_info("P/F.framework"), None)
        self.assertEqual(framework.framework_info("P/F.framework/_debug"), None)
        self.assertEqual(
            framework.framework_info("P/F.framework/F"), d("P", "F.framework/F", "F")
        )
        self.assertEqual(
            framework.framework_info("P/F.framework/F_debug"),
            d("P", "F.framework/F_debug", "F", suffix="debug"),
        )
        self.assertEqual(framework.framework_info("P/F.framework/Versions"), None)
        self.assertEqual(framework.framework_info("P/F.framework/Versions/A"), None)
        self.assertEqual(
            framework.framework_info("P/F.framework/Versions/A/F"),
            d("P", "F.framework/Versions/A/F", "F", "A"),
        )
        self.assertEqual(
            framework.framework_info("P/F.framework/Versions/A/F_debug"),
            d("P", "F.framework/Versions/A/F_debug", "F", "A", "debug"),
        )


if __name__ == "__main__":
    unittest.main()



"""
Internal helpers for basic commandline tools
"""
from __future__ import absolute_import, print_function

import os
import sys

from macholib.util import is_platform_file


def check_file(fp, path, callback):
    if not os.path.exists(path):
        print(
            "%s: %s: No such file or directory" % (sys.argv[0], path), file=sys.stderr
        )
        return 1

    try:
        is_plat = is_platform_file(path)

    except IOError as msg:
        print("%s: %s: %s" % (sys.argv[0], path, msg), file=sys.stderr)
        return 1

    else:
        if is_plat:
            callback(fp, path)
    return 0


def main(callback):
    args = sys.argv[1:]
    name = os.path.basename(sys.argv[0])
    err = 0

    if not args:
        print("Usage: %s filename..." % (name,), file=sys.stderr)
        return 1

    for base in args:
        if os.path.isdir(base):
            for root, _dirs, files in os.walk(base):
                for fn in files:
                    err |= check_file(sys.stdout, os.path.join(root, fn), callback)
        else:
            err |= check_file(sys.stdout, base, callback)

    return err



import sys

from macholib import mach_o

if sys.version_info[:2] <= (2, 6):
    import unittest2 as unittest
else:
    import unittest


class TestMachO(unittest.TestCase):
    # This module is just a set of struct definitions,
    # not sure how to test those without replicating
    # the code.
    #
    # The definitions will get exercised by the
    # other tests, therefore testing is ignored
    # for now.

    def test_consistency(self):
        self.assertEqual(
            mach_o.MH_FLAGS_DESCRIPTIONS.keys(), mach_o.MH_FLAGS_NAMES.keys()
        )
        for k in dir(mach_o):
            if not k.startswith("MH_"):
                continue
            v = getattr(mach_o, k)
            if isinstance(v, int) and any(v == 1 << x for x in range(31)):
                self.assertTrue(
                    v in mach_o.MH_FLAGS_NAMES, "No NAME and DESCRIPTION for %s" % (k,)
                )


if __name__ == "__main__":
    unittest.main()



import functools
import os
import sys

from macholib import dyld

if sys.version_info[:2] <= (2, 6):
    import unittest2 as unittest
else:
    import unittest


class DyldPatcher(object):
    def __init__(self):
        self.calls = []
        self.patched = {}

    def clear_calls(self):
        self.calls = []

    def cleanup(self):
        for name in self.patched:
            setattr(dyld, name, self.patched[name])

    def log_calls(self, name):
        if name in self.patched:
            return

        self.patched[name] = getattr(dyld, name)

        @functools.wraps(self.patched[name])
        def wrapper(*args, **kwds):
            self.calls.append((name, args, kwds))
            return self.patched[name](*args, **kwds)

        setattr(dyld, name, wrapper)


class TestDyld(unittest.TestCase):
    if not hasattr(unittest.TestCase, "assertIsInstance"):

        def assertIsInstance(self, value, types, message=None):
            self.assertTrue(
                isinstance(value, types),
                message or "%r is not an instance of %r" % (value, types),
            )

    def setUp(self):
        self._environ = os.environ
        os.environ = dict([(k, os.environ[k]) for k in os.environ if "DYLD" not in k])
        self._dyld_env = dyld._dyld_env
        self._dyld_image_suffix = dyld.dyld_image_suffix

    def tearDown(self):
        dyld._dyld_env = self._dyld_env
        dyld.dyld_image_suffix = self._dyld_image_suffix
        os.environ = self._environ

    if sys.version_info[0] == 2:

        def test_ensure_utf8(self):
            self.assertEqual(dyld._ensure_utf8("hello"), "hello")
            self.assertEqual(dyld._ensure_utf8("hello".decode("utf-8")), "hello")
            self.assertEqual(dyld._ensure_utf8(None), None)

    else:

        def test_ensure_utf8(self):
            self.assertEqual(dyld._ensure_utf8("hello"), "hello")
            self.assertEqual(dyld._ensure_utf8(None), None)
            self.assertRaises(ValueError, dyld._ensure_utf8, b"hello")

    def test__dyld_env(self):
        new = os.environ

        self.assertEqual(dyld._dyld_env(None, "DYLD_FOO"), [])
        self.assertEqual(dyld._dyld_env({"DYLD_FOO": "bar"}, "DYLD_FOO"), ["bar"])
        self.assertEqual(
            dyld._dyld_env({"DYLD_FOO": "bar:baz"}, "DYLD_FOO"), ["bar", "baz"]
        )
        self.assertEqual(dyld._dyld_env({}, "DYLD_FOO"), [])
        self.assertEqual(dyld._dyld_env({"DYLD_FOO": ""}, "DYLD_FOO"), [])
        os.environ["DYLD_FOO"] = "foobar"
        self.assertEqual(dyld._dyld_env(None, "DYLD_FOO"), ["foobar"])
        os.environ["DYLD_FOO"] = "foobar:nowhere"
        self.assertEqual(dyld._dyld_env(None, "DYLD_FOO"), ["foobar", "nowhere"])
        self.assertEqual(dyld._dyld_env({"DYLD_FOO": "bar"}, "DYLD_FOO"), ["bar"])
        self.assertEqual(dyld._dyld_env({}, "DYLD_FOO"), [])

        self.assertEqual(dyld.dyld_image_suffix(), None)
        self.assertEqual(dyld.dyld_image_suffix(None), None)
        self.assertEqual(dyld.dyld_image_suffix({"DYLD_IMAGE_SUFFIX": "bar"}), "bar")
        os.environ["DYLD_IMAGE_SUFFIX"] = "foobar"
        self.assertEqual(dyld.dyld_image_suffix(), "foobar")
        self.assertEqual(dyld.dyld_image_suffix(None), "foobar")

    def test_dyld_helpers(self):
        record = []

        def fake__dyld_env(env, key):
            record.append((env, key))
            return ["hello"]

        dyld._dyld_env = fake__dyld_env
        self.assertEqual(dyld.dyld_framework_path(), ["hello"])
        self.assertEqual(dyld.dyld_framework_path({}), ["hello"])

        self.assertEqual(dyld.dyld_library_path(), ["hello"])
        self.assertEqual(dyld.dyld_library_path({}), ["hello"])

        self.assertEqual(dyld.dyld_fallback_framework_path(), ["hello"])
        self.assertEqual(dyld.dyld_fallback_framework_path({}), ["hello"])

        self.assertEqual(dyld.dyld_fallback_library_path(), ["hello"])
        self.assertEqual(dyld.dyld_fallback_library_path({}), ["hello"])

        self.assertEqual(
            record,
            [
                (None, "DYLD_FRAMEWORK_PATH"),
                ({}, "DYLD_FRAMEWORK_PATH"),
                (None, "DYLD_LIBRARY_PATH"),
                ({}, "DYLD_LIBRARY_PATH"),
                (None, "DYLD_FALLBACK_FRAMEWORK_PATH"),
                ({}, "DYLD_FALLBACK_FRAMEWORK_PATH"),
                (None, "DYLD_FALLBACK_LIBRARY_PATH"),
                ({}, "DYLD_FALLBACK_LIBRARY_PATH"),
            ],
        )

    def test_dyld_suffix_search(self):
        envs = [object()]

        def fake_suffix(env):
            envs[0] = env
            return None

        dyld.dyld_image_suffix = fake_suffix

        iterator = ["/usr/lib/foo", "/usr/lib/foo.dylib"]
        result = dyld.dyld_image_suffix_search(iter(iterator))
        self.assertEqual(list(result), iterator)
        self.assertEqual(envs[0], None)

        result = dyld.dyld_image_suffix_search(iter(iterator), {})
        self.assertEqual(list(result), iterator)
        self.assertEqual(envs[0], {})

        envs = [object()]

        def fake_suffix(env):
            envs[0] = env
            return "_profile"

        dyld.dyld_image_suffix = fake_suffix

        iterator = ["/usr/lib/foo", "/usr/lib/foo.dylib"]
        result = dyld.dyld_image_suffix_search(iter(iterator))
        self.assertEqual(
            list(result),
            [
                "/usr/lib/foo_profile",
                "/usr/lib/foo",
                "/usr/lib/foo_profile.dylib",
                "/usr/lib/foo.dylib",
            ],
        )
        self.assertEqual(envs[0], None)

        result = dyld.dyld_image_suffix_search(iter(iterator), {})
        self.assertEqual(
            list(result),
            [
                "/usr/lib/foo_profile",
                "/usr/lib/foo",
                "/usr/lib/foo_profile.dylib",
                "/usr/lib/foo.dylib",
            ],
        )
        self.assertEqual(envs[0], {})

    def test_override_search(self):
        os.environ["DYLD_FRAMEWORK_PATH"] = ""
        os.environ["DYLD_LIBRARY_PATH"] = ""

        self.assertEqual(list(dyld.dyld_override_search("foo.dyld", None)), [])
        self.assertEqual(
            list(dyld.dyld_override_search("/usr/lib/libfoo.dyld", None)), []
        )
        self.assertEqual(
            list(
                dyld.dyld_override_search(
                    "/Library/Frameworks/Python.framework/Versions/Current/Python", None
                )
            ),
            [],
        )

        os.environ["DYLD_FRAMEWORK_PATH"] = "/Foo/Frameworks:/Bar/Frameworks"
        os.environ["DYLD_LIBRARY_PATH"] = ""
        self.assertEqual(list(dyld.dyld_override_search("foo.dyld", None)), [])
        self.assertEqual(
            list(dyld.dyld_override_search("/usr/lib/libfoo.dyld", None)), []
        )
        self.assertEqual(
            list(
                dyld.dyld_override_search(
                    "/Library/Frameworks/Python.framework/Versions/Current/Python", None
                )
            ),
            [
                "/Foo/Frameworks/Python.framework/Versions/Current/Python",
                "/Bar/Frameworks/Python.framework/Versions/Current/Python",
            ],
        )

        os.environ["DYLD_FRAMEWORK_PATH"] = ""
        os.environ["DYLD_LIBRARY_PATH"] = "/local/lib:/remote/lib"
        self.assertEqual(
            list(dyld.dyld_override_search("foo.dyld", None)),
            ["/local/lib/foo.dyld", "/remote/lib/foo.dyld"],
        )
        self.assertEqual(
            list(dyld.dyld_override_search("/usr/lib/libfoo.dyld", None)),
            ["/local/lib/libfoo.dyld", "/remote/lib/libfoo.dyld"],
        )
        self.assertEqual(
            list(
                dyld.dyld_override_search(
                    "/Library/Frameworks/Python.framework/Versions/Current/Python", None
                )
            ),
            ["/local/lib/Python", "/remote/lib/Python"],
        )

        os.environ["DYLD_FRAMEWORK_PATH"] = "/Foo/Frameworks:/Bar/Frameworks"
        os.environ["DYLD_LIBRARY_PATH"] = "/local/lib:/remote/lib"
        self.assertEqual(
            list(dyld.dyld_override_search("foo.dyld", None)),
            ["/local/lib/foo.dyld", "/remote/lib/foo.dyld"],
        )
        self.assertEqual(
            list(dyld.dyld_override_search("/usr/lib/libfoo.dyld", None)),
            ["/local/lib/libfoo.dyld", "/remote/lib/libfoo.dyld"],
        )
        self.assertEqual(
            list(
                dyld.dyld_override_search(
                    "/Library/Frameworks/Python.framework/Versions/Current/Python", None
                )
            ),
            [
                "/Foo/Frameworks/Python.framework/Versions/Current/Python",
                "/Bar/Frameworks/Python.framework/Versions/Current/Python",
                "/local/lib/Python",
                "/remote/lib/Python",
            ],
        )

    def test_executable_path_search(self):
        self.assertEqual(
            list(dyld.dyld_executable_path_search("/usr/lib/foo.dyld", "/usr/bin")), []
        )
        self.assertEqual(
            list(
                dyld.dyld_executable_path_search(
                    "@executable_path/foo.dyld", "/usr/bin"
                )
            ),
            ["/usr/bin/foo.dyld"],
        )
        self.assertEqual(
            list(
                dyld.dyld_executable_path_search(
                    "@executable_path/../../lib/foo.dyld", "/usr/bin"
                )
            ),
            ["/usr/bin/../../lib/foo.dyld"],
        )

    def test_default_search(self):
        self.assertEqual(
            list(dyld.dyld_default_search("/usr/lib/mylib.dylib", None)),
            [
                "/usr/lib/mylib.dylib",
                os.path.join(os.path.expanduser("~/lib"), "mylib.dylib"),
                "/usr/local/lib/mylib.dylib",
                "/lib/mylib.dylib",
                "/usr/lib/mylib.dylib",
            ],
        )

        self.assertEqual(
            list(
                dyld.dyld_default_search(
                    "/Library/Frameworks/Python.framework/Versions/2.7/Python", None
                )
            ),
            [
                "/Library/Frameworks/Python.framework/Versions/2.7/Python",
                os.path.join(
                    os.path.expanduser("~/Library/Frameworks"),
                    "Python.framework/Versions/2.7/Python",
                ),
                "/Library/Frameworks/Python.framework/Versions/2.7/Python",
                "/Network/Library/Frameworks/Python.framework/Versions/2.7/Python",
                "/System/Library/Frameworks/Python.framework/Versions/2.7/Python",
                os.path.join(os.path.expanduser("~/lib"), "Python"),
                "/usr/local/lib/Python",
                "/lib/Python",
                "/usr/lib/Python",
            ],
        )

        os.environ["DYLD_FALLBACK_LIBRARY_PATH"] = "/local/lib:/network/lib"
        os.environ["DYLD_FALLBACK_FRAMEWORK_PATH"] = ""

        self.assertEqual(
            list(dyld.dyld_default_search("/usr/lib/mylib.dylib", None)),
            [
                "/usr/lib/mylib.dylib",
                "/local/lib/mylib.dylib",
                "/network/lib/mylib.dylib",
            ],
        )

        self.assertEqual(
            list(
                dyld.dyld_default_search(
                    "/Library/Frameworks/Python.framework/Versions/2.7/Python", None
                )
            ),
            [
                "/Library/Frameworks/Python.framework/Versions/2.7/Python",
                os.path.join(
                    os.path.expanduser("~/Library/Frameworks"),
                    "Python.framework/Versions/2.7/Python",
                ),
                "/Library/Frameworks/Python.framework/Versions/2.7/Python",
                "/Network/Library/Frameworks/Python.framework/Versions/2.7/Python",
                "/System/Library/Frameworks/Python.framework/Versions/2.7/Python",
                "/local/lib/Python",
                "/network/lib/Python",
            ],
        )

        os.environ["DYLD_FALLBACK_LIBRARY_PATH"] = ""
        os.environ["DYLD_FALLBACK_FRAMEWORK_PATH"] = "/MyFrameworks:/OtherFrameworks"

        self.assertEqual(
            list(dyld.dyld_default_search("/usr/lib/mylib.dylib", None)),
            [
                "/usr/lib/mylib.dylib",
                os.path.join(os.path.expanduser("~/lib"), "mylib.dylib"),
                "/usr/local/lib/mylib.dylib",
                "/lib/mylib.dylib",
                "/usr/lib/mylib.dylib",
            ],
        )

        self.assertEqual(
            list(
                dyld.dyld_default_search(
                    "/Library/Frameworks/Python.framework/Versions/2.7/Python", None
                )
            ),
            [
                "/Library/Frameworks/Python.framework/Versions/2.7/Python",
                "/MyFrameworks/Python.framework/Versions/2.7/Python",
                "/OtherFrameworks/Python.framework/Versions/2.7/Python",
                os.path.join(os.path.expanduser("~/lib"), "Python"),
                "/usr/local/lib/Python",
                "/lib/Python",
                "/usr/lib/Python",
            ],
        )

        os.environ["DYLD_FALLBACK_LIBRARY_PATH"] = "/local/lib:/network/lib"
        os.environ["DYLD_FALLBACK_FRAMEWORK_PATH"] = "/MyFrameworks:/OtherFrameworks"

        self.assertEqual(
            list(dyld.dyld_default_search("/usr/lib/mylib.dylib", None)),
            [
                "/usr/lib/mylib.dylib",
                "/local/lib/mylib.dylib",
                "/network/lib/mylib.dylib",
            ],
        )

        self.assertEqual(
            list(
                dyld.dyld_default_search(
                    "/Library/Frameworks/Python.framework/Versions/2.7/Python", None
                )
            ),
            [
                "/Library/Frameworks/Python.framework/Versions/2.7/Python",
                "/MyFrameworks/Python.framework/Versions/2.7/Python",
                "/OtherFrameworks/Python.framework/Versions/2.7/Python",
                "/local/lib/Python",
                "/network/lib/Python",
            ],
        )

    def test_dyld_find(self):
        result = dyld.dyld_find("/usr/lib/libSystem.dylib")
        self.assertEqual(result, "/usr/lib/libSystem.dylib")
        self.assertIsInstance(result, str)  # bytes on 2.x, unicode on 3.x

        result = dyld.dyld_find(b"/usr/lib/libSystem.dylib".decode("ascii"))
        self.assertEqual(result, "/usr/lib/libSystem.dylib")
        self.assertIsInstance(result, str)  # bytes on 2.x, unicode on 3.x

        patcher = DyldPatcher()
        try:
            patcher.log_calls("dyld_image_suffix_search")
            patcher.log_calls("dyld_override_search")
            patcher.log_calls("dyld_executable_path_search")
            patcher.log_calls("dyld_default_search")

            result = dyld.dyld_find("/usr/lib/libSystem.dylib")
            self.assertEqual(
                patcher.calls[:-1],
                [
                    ("dyld_override_search", ("/usr/lib/libSystem.dylib", None), {}),
                    (
                        "dyld_executable_path_search",
                        ("/usr/lib/libSystem.dylib", None),
                        {},
                    ),
                    ("dyld_default_search", ("/usr/lib/libSystem.dylib", None), {}),
                ],
            )
            self.assertEqual(patcher.calls[-1][0], "dyld_image_suffix_search")
            patcher.clear_calls()

            result = dyld.dyld_find("/usr/lib/libSystem.dylib", env=None)
            self.assertEqual(
                patcher.calls[:-1],
                [
                    ("dyld_override_search", ("/usr/lib/libSystem.dylib", None), {}),
                    (
                        "dyld_executable_path_search",
                        ("/usr/lib/libSystem.dylib", None),
                        {},
                    ),
                    ("dyld_default_search", ("/usr/lib/libSystem.dylib", None), {}),
                ],
            )
            self.assertEqual(patcher.calls[-1][0], "dyld_image_suffix_search")
            patcher.clear_calls()

            result = dyld.dyld_find("/usr/lib/libSystem.dylib", env={})
            self.assertEqual(
                patcher.calls[:-1],
                [
                    ("dyld_override_search", ("/usr/lib/libSystem.dylib", {}), {}),
                    (
                        "dyld_executable_path_search",
                        ("/usr/lib/libSystem.dylib", None),
                        {},
                    ),
                    ("dyld_default_search", ("/usr/lib/libSystem.dylib", {}), {}),
                ],
            )
            self.assertEqual(patcher.calls[-1][0], "dyld_image_suffix_search")
            patcher.clear_calls()

            result = dyld.dyld_find(
                "/usr/lib/libSystem.dylib", executable_path="/opt/py2app/bin", env={}
            )
            self.assertEqual(
                patcher.calls[:-1],
                [
                    ("dyld_override_search", ("/usr/lib/libSystem.dylib", {}), {}),
                    (
                        "dyld_executable_path_search",
                        ("/usr/lib/libSystem.dylib", "/opt/py2app/bin"),
                        {},
                    ),
                    ("dyld_default_search", ("/usr/lib/libSystem.dylib", {}), {}),
                ],
            )
            self.assertEqual(patcher.calls[-1][0], "dyld_image_suffix_search")
            patcher.clear_calls()

        finally:
            patcher.cleanup()

    def test_framework_find(self):
        result = dyld.framework_find(
            "/System/Library/Frameworks/Cocoa.framework/Versions/Current/Cocoa"
        )
        self.assertEqual(
            result, "/System/Library/Frameworks/Cocoa.framework/Versions/Current/Cocoa"
        )
        self.assertIsInstance(result, str)  # bytes on 2.x, unicode on 3.x

        result = dyld.framework_find(
            b"/System/Library/Frameworks/Cocoa.framework/Versions/Current/Cocoa".decode(
                "latin1"
            )
        )
        self.assertEqual(
            result, "/System/Library/Frameworks/Cocoa.framework/Versions/Current/Cocoa"
        )
        self.assertIsInstance(result, str)  # bytes on 2.x, unicode on 3.x

        result = dyld.framework_find("Cocoa.framework")
        self.assertEqual(result, "/System/Library/Frameworks/Cocoa.framework/Cocoa")
        self.assertIsInstance(result, str)  # bytes on 2.x, unicode on 3.x

        result = dyld.framework_find("Cocoa")
        self.assertEqual(result, "/System/Library/Frameworks/Cocoa.framework/Cocoa")
        self.assertIsInstance(result, str)  # bytes on 2.x, unicode on 3.x

        patcher = DyldPatcher()
        try:
            patcher.log_calls("dyld_find")

            result = dyld.framework_find(
                "/System/Library/Frameworks/Cocoa.framework/Versions/Current/Cocoa"
            )
            self.assertEqual(
                patcher.calls,
                [
                    (
                        "dyld_find",
                        (
                            "/System/Library/Frameworks/Cocoa.framework/Versions/Current/Cocoa",
                        ),
                        {"env": None, "executable_path": None},
                    )
                ],
            )
            patcher.clear_calls()

            result = dyld.framework_find("Cocoa")
            self.assertEqual(
                patcher.calls,
                [
                    ("dyld_find", ("Cocoa",), {"env": None, "executable_path": None}),
                    (
                        "dyld_find",
                        ("Cocoa.framework/Cocoa",),
                        {"env": None, "executable_path": None},
                    ),
                ],
            )
            patcher.clear_calls()

            result = dyld.framework_find("Cocoa", "/my/sw/bin", {})
            self.assertEqual(
                patcher.calls,
                [
                    (
                        "dyld_find",
                        ("Cocoa",),
                        {"env": {}, "executable_path": "/my/sw/bin"},
                    ),
                    (
                        "dyld_find",
                        ("Cocoa.framework/Cocoa",),
                        {"env": {}, "executable_path": "/my/sw/bin"},
                    ),
                ],
            )
            patcher.clear_calls()

        finally:
            patcher.cleanup()


class TestTrivialDyld(unittest.TestCase):
    # Tests ported from the implementation file
    def testBasic(self):
        self.assertEqual(dyld.dyld_find("libSystem.dylib"), "/usr/lib/libSystem.dylib")
        self.assertEqual(
            dyld.dyld_find("System.framework/System"),
            "/System/Library/Frameworks/System.framework/System",
        )


if __name__ == "__main__":
    unittest.main()
